package zap;

import VASSAL.command.ChangePiece;
import VASSAL.command.Command;
import VASSAL.counters.*;
import VASSAL.Info;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.AffineTransform;

public class Magnifier extends Decorator implements EditablePiece {
  // This should uniquely identify this type of Decorator
  public static final String ID = "mag;";

  // The magnification factor
  private double mag = 1.0;

  // The minimum and maximum magnification
  private double minMag = 0.5, maxMag = 2.0;

  // The keyboard command to magnify the counter.  This is CTRL-M
  private KeyStroke magnifyCommand =
      KeyStroke.getKeyStroke('M', java.awt.event.InputEvent.CTRL_MASK);

  // The list of popup menu commands
  private KeyCommand[] commands;

  // Construct a new Magnifier
  public Magnifier(String type, GamePiece inner) {
    mySetType(type);
    setInner(inner);
  }

  // A no-arg constructor is necessary to import the counter
  // into a module with the game piece designer dialog
  public Magnifier() {
    this(ID + "0.5;2.0", null);
  }

  // The name of the counter
  public String getName() {
    return piece.getName();
  }

  // It's important that myGetType() returns the same
  // value that was used in the constructor
  public String myGetType() {
    return ID + minMag + ";" + maxMag;
  }

  // The state of the counter is the magnification factor
  public String myGetState() {
    return "" + mag;
  }

  // Set the magnification factor
  public void mySetState(String s) {
    try {
      mag = Double.valueOf(s).doubleValue();
    }
    catch (NumberFormatException ex) {
      ex.printStackTrace();
      mag = 1.0;
    }
  }

  // Lazily create the menu commands
  public KeyCommand[] myGetKeyCommands() {
    if (commands == null) {
      commands = new KeyCommand[]{new KeyCommand("Magnify", magnifyCommand, this)};
    }
    return commands;
  }

  // Respond to a keyboard command
  public Command myKeyEvent(KeyStroke stroke) {
    Command c = null;
    if (magnifyCommand.equals(stroke)) {
      GamePiece target = Decorator.getOutermost(this);
      String oldState = target.getState();
      // Prompt user for magnification factor
      String s = JOptionPane.showInputDialog("Enter magnification:");
      if (s != null) {
        try {
          mag = Double.valueOf(s).doubleValue();
          mag = Math.max(mag, minMag);
          mag = Math.min(mag, maxMag);
          c = new ChangePiece(target.getId(), oldState, target.getState());
        }
        catch (NumberFormatException ex) {
        }
      }
    }
    return c;
  }

  // Draw the piece
  public void draw(Graphics g, int x, int y, Component obs, double zoom) {
    piece.draw(g, x, y, obs, zoom * mag);
  }

  // This is the area that should be redrawn after the counter has changed
  public Rectangle boundingBox() {
    Rectangle r = piece.boundingBox();
    r.width *= mag;
    r.height *= mag;
    r.translate(-r.width / 2 + piece.boundingBox().width / 2,
                -r.height / 2 + piece.boundingBox().height / 2);
    return r;
  }

  // This is the rectangle that the user must click inside
  // to select the counter
  public Shape getShape() {
    if (Info.is2dEnabled()) {
      return AffineTransform.getScaleInstance(mag,mag).createTransformedShape(piece.getShape());
    }
    else {
      Rectangle r = piece.getShape().getBounds();
      r.width *= mag;
      r.height *= mag;
      r.x *= mag;  //  Because getShape() returns a shape centered at (0,0), we can simply scale the position as well
      r.y *= mag;
      return r;
    }
  }

  /* -- The following methods allow the piece to be used in the
     game piece designer dialog */

  // The trait name corresponding to this class
  public String getDescription() {
    return "Magnifyable";
  }

  // Set the type information
  public void mySetType(String type) {
    type = type.substring(ID.length());
    int i = type.indexOf(";");
    try {
      minMag = Double.valueOf(type.substring(0, i)).doubleValue();
    }
    catch (NumberFormatException ex) {
      ex.printStackTrace();
    }
    try {
      maxMag = Double.valueOf(type.substring(i + 1)).doubleValue();
    }
    catch (NumberFormatException ex) {
      ex.printStackTrace();
    }
  }

  // Not necessary to provide a help file
  public VASSAL.build.module.documentation.HelpFile getHelpFile() {
    return null;
  }

  public PieceEditor getEditor() {
    return new SimplePieceEditor(this);
  }
}
